/*
 * Copyright (C) 2015 by Multi-Tech Systems
 *
 * This file is part of libmts.
 *
 * libmts is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * libmts is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libmts.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*! \file   MTS_AutoLock.h
 \brief  Auto lock
 \date   08OCT14
 \author Pavlo Samko

 Auto lock.
 */


#ifndef _MTS_AUTOLOCK_H_
#define _MTS_AUTOLOCK_H_

#include <mts/MTS_NonCopyable.h>
#include <mts/MTS_Lock.h>

namespace MTS {

    //! Auto lock
    /*!
     Auto lock

     \sa Lock
     */

    class AutoLock: private NonCopyable {

        public:
            AutoLock(Lock &lock): m_lock(lock) { m_lock.lock(); }   //!< Constructs the AutoLock
            ~AutoLock() { m_lock.unlock(); }                        //!< Destructs the AutoLock

        private:
            Lock &m_lock;                                           //!< Lock
    };
}

#endif
